$regfile = "attiny2313.dat"                                 ' specify the used micro
$crystal = 12000000                                         ' used crystal frequency
$baud = 19200
$hwstack = 32                                               ' default use 32 for the hardware stack
$swstack = 24                                               ' default use 10 for the SW stack
$framesize = 40                                             ' default use 40 for the frame space

'configs
Config Portb = &B00011111                                   'PB0,1,2,4 ulazi, ostalo izlazi
Portb = &B11100000                                          'pullup na ulazima
Config Portd = &B11100011                                   'PD2,3,4 - inputs
Portd = &B00011100                                          'pullup na ulazima
Config Debounce = 30                                        'config debounce to 30ms
Config Timer1 = Timer , Prescale = 256 , Compare A = Disconnect , Compare B = Disconnect , Clear Timer = 1
Config Int0 = Low Level                                     'needed to wake up MCU from powerdown

'define port pin
Speaker Alias Portb.0                                       'zvucnik
Relay Alias Portb.1                                         'relej
Clock Alias Portb.2                                         '-------------------------------
Dat Alias Portb.3                                           'upravljacki pinovi za registre
Store Alias Portb.4                                         '-------------------------------

S1 Alias Pind.2                                             '-------------------------------
S2 Alias Pind.3                                             'tipke
S3 Alias Pind.4                                             '-------------------------------

'def flags
Dim Check_keys_flag As Bit
Dim Disp_update_flag As Bit
Dim Middle_leds_flag As Bit
Dim Timer1_toggle_flag As Bit
Dim Make_noise_flag As Bit
Dim Shutdown_flag As Bit
Dim Read_eeprom_flag As Bit
Dim Write_eeprom_flag As Bit
'define variables
Dim I As Byte , J As Byte
Dim Empty As Byte
Dim Disp(4) As Byte
Dim Value(4) As Byte
Dim Idlee As Byte

'define interrupts
Disable Int0
Enable Compare1a
Enable Interrupts
On Compare1a Timer1_int
Compare1a = 23436                                           'za 12MHz kristal i prescale=256 -> ISR svakih 0.5s
Start Timer1

'define constants
Const Pulses = 400
Const Periods = 333
Set Read_eeprom_flag



'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Glavni program
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Do

Debounce S1 , 0 , Key1 , Sub
If Check_keys_flag = 0 Then
   Debounce S2 , 0 , Key2 , Sub
   Debounce S3 , 0 , Key3 , Sub
End If

If Write_eeprom_flag = 1 Then Gosub Write_eeprom
If Disp_update_flag = 1 Then Gosub Disp_update
If Make_noise_flag = 1 Then Gosub Make_noise
If Read_eeprom_flag = 1 Then Gosub Read_eeprom
If Shutdown_flag = 1 Then Gosub Shutdown

Loop
End



'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Tipke
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Key1:
Set Disp_update_flag
Set Write_eeprom_flag
Toggle Check_keys_flag                                      'onemoguci postavljanje vremena dok je u tijeku odbrojavanje
Toggle Timer1_toggle_flag
Toggle Relay
Return

Key2:
Decr Value(2)                                               'namjestanje vremena u koraku od 10s
If Value(1) <> 0 Then Value(1) = 10
Set Disp_update_flag
Return

Key3:
Incr Value(2)                                               'namjestanje vremena u koraku od 10s
If Value(1) <> 0 Then Value(1) = 0
Set Disp_update_flag
Return




'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Osvjezi display
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Disp_update:
If Value(1) > 9 And Value(1) < 100 Then                     '-------------------------------
   Value(1) = 0                                             '-                              -
   Incr Value(2)                                            '-                              -
End If                                                      '-                              -
If Value(1) > 100 Then                                      '-                              -
   Value(1) = 9                                             '-                              -
   Decr Value(2)                                            '-                              -
End If                                                      '-                              -
If Value(2) > 5 And Value(2) < 100 Then                     '-                              -
   Value(2) = 0                                             '-                              -
   Incr Value(3)                                            '-                              -
End If                                                      'provjera pojedinih vrijednosti -
If Value(2) > 100 Then                                      '-                              -
   Value(2) = 5                                             '-                              -
   Decr Value(3)                                            '-                              -
End If                                                      '-                              -
If Value(3) > 9 And Value(3) < 100 Then                     '-                              -
   Value(3) = 0                                             '-                              -
   Incr Value(4)                                            '-                              -
End If                                                      '-                              -
If Value(3) > 100 Then                                      '-                              -
   Value(3) = 9                                             '-                              -
   Decr Value(4)                                            '-                              -
End If                                                      '-                              -
If Value(4) > 9 And Value(4) < 100 Then Value(4) = 0        '-                              -
If Value(4) > 100 Then Value(4) = 9                         '--------------------------------

For I = 1 To 4
Disp(i) = Lookup(value(i) , Table)                          'cita vrijednosti za 4 displeja definirane u tablici
Next I

If Timer1_toggle_flag = 0 Then Set Middle_leds_flag         'omoguci stalno upaljenu dvotocku ako nije ukljuceno odbrojavanje
If Middle_leds_flag = 1 Then Disp(3) = Disp(3) + 16         'ukljuci dvotocku koja je spojena na DP od treceg displeja
If Value(4) = 0 Then Disp(4) = 0                            'ugasi display 4 ako mu je vrijednost 0

For I = 1 To 4
Shiftout Dat , Clock , Disp(i) , 1                          'napuni registre podacima
Next I

Set Store                                                   'prebaci podatke iz registara na izlaze cipa
Reset Store
Reset Disp_update_flag
Idlee = 0                                                   'ako se vrijednost displeja mjenja, znaci
Return                                                      'uredaj nije u stanju mirovanja




'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Alarm
'zvuci otprilike ovako
'BIP-BIP-BIP-BIP  BIP-BIP-BIP-BIP  BIP-BIP-BIP-BIP
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Make_noise:
For J = 0 To 2
For I = 0 To 3
Sound Speaker , Pulses , Periods
Waitms 10
Next I
Waitms 500
Next J
Set Read_eeprom_flag
Reset Make_noise_flag
Return



'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Citanje i pisanje iz EEPROM-a
'sprema i cita sa lokacija 1-4
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Read_eeprom:
For I = 1 To 4
Readeeprom Value(i) , I
Next I
Set Disp_update_flag
Reset Read_eeprom_flag
Return

Write_eeprom:
For I = 1 To 4
Writeeeprom Value(i) , I
Next I
Reset Write_eeprom_flag
Return




'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Gasenje mikrokontrolera
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Shutdown:
For I = 0 To 3
Shiftout Dat , Clock , Empty , 1                            'isprazni registre, tj. napuni ih nulama
Next I
Set Store                                                   'prebaci podatke iz registara na izlaze cipa (displeji ce biti ugaseni)
Reset Store
Enable Int0                                                 'omoguci Interrupt0 kako bi CPU tipkom 1 mogli probuditi
Stop Timer1                                                 'zaustavi timer1 kako on nebi probudio CPU
Powerdown                                                   'gasi
Bitwait S1 , Set                                            'pritiskom na tipku 1 CPU se ukljucuje, zatim cekaj sve dok se tipka ne otpusti
Disable Int0
Start Timer1
Set Read_eeprom_flag
Reset Shutdown_flag
Return




'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Timer1 prekidna rutina
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Timer1_int:
Incr Idlee
If Timer1_toggle_flag = 1 Then
      If Value(1) = 0 And Value(2) = 0 And Value(3) = 0 And Value(4) = 0 Then       'ako je vrijednost 00:00 onda...
         Reset Timer1_toggle_flag
         Reset Relay
         Reset Check_keys_flag
         Set Make_noise_flag
      End If
   Toggle Middle_leds_flag
   Set Disp_update_flag
   If Middle_leds_flag = 1 Then Decr Value(1)
End If

If Idlee = 240 Then                                         'ako je CPU neaktivan 2min...
   Set Write_eeprom_flag                                    'zapisi stanje timera
   Set Shutdown_flag                                        'zatim ugasi CPU
End If
Return




'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
'------------------------------------------------------------------------------
'Tablica brojeva (0-9) za 7-seg display
'------------------------------------------------------------------------------
'++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Table:
Data 235 , 96 , 199 , 230 , 108 , 174 , 175 , 224 , 239 , 238
